//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Particle/FormFactorComposition.h
//! @brief     Defines class FormFactorComposition.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef USER_API
#ifndef BORNAGAIN_SAMPLE_PARTICLE_FORMFACTORCOMPOSITION_H
#define BORNAGAIN_SAMPLE_PARTICLE_FORMFACTORCOMPOSITION_H

#include "Sample/Scattering/IFormFactor.h"

//! Coherent sum of different scalar IFormFactor%s.
//!
//! Used by ParticleComposition.
//! If same particles are at different positions, then consider
//! FormFactorDecoratorMultiPositionFactor (restore from commit 0500a26de76).

class FormFactorComposition : public IFormFactor {
public:
    std::string className() const final { return "FormFactorComposition"; }

    FormFactorComposition() = default;
    ~FormFactorComposition() override;

    FormFactorComposition* clone() const override;

    double radialExtension() const override;

    double bottomZ(const IRotation* rotation) const override;

    double topZ(const IRotation* rotation) const override;

    void addFormFactor(const IFormFactor& formfactor);

    void setAmbientMaterial(const Material& material) override;

    complex_t theFF(const WavevectorInfo& wavevectors) const override;

#ifndef SWIG
    //! Calculates and returns a polarized form factor calculation in DWBA
    Eigen::Matrix2cd thePolFF(const WavevectorInfo& wavevectors) const override;
#endif

protected:
    std::vector<IFormFactor*> m_formfactors;
};

#endif // BORNAGAIN_SAMPLE_PARTICLE_FORMFACTORCOMPOSITION_H
#endif // USER_API
