//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Scattering/FormFactorDecoratorRotation.h
//! @brief     Defines class FormFactorDecoratorRotation.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif

#ifndef USER_API
#ifndef BORNAGAIN_SAMPLE_SCATTERING_FORMFACTORDECORATORROTATION_H
#define BORNAGAIN_SAMPLE_SCATTERING_FORMFACTORDECORATORROTATION_H

#include "Sample/Scattering/DecoratedFF.h"

class IRotation;
class RotMatrix;

//! Equips a form factor with a rotation.

class FormFactorDecoratorRotation : public DecoratedFF {
public:
    FormFactorDecoratorRotation(const IDecoratableBorn& ff, const RotMatrix& rotMatrix);

    ~FormFactorDecoratorRotation();

    FormFactorDecoratorRotation* clone() const override;
    std::string className() const final { return "FormFactorDecoratorRotation"; }

    double bottomZ(const IRotation* rotation) const override;

    double topZ(const IRotation* rotation) const override;

    complex_t theFF(const WavevectorInfo& wavevectors) const override;
#ifndef SWIG
    Eigen::Matrix2cd thePolFF(const WavevectorInfo& wavevectors) const override;
#endif

private:
    std::unique_ptr<RotMatrix> m_rotMatrix;
};

#endif // BORNAGAIN_SAMPLE_SCATTERING_FORMFACTORDECORATORROTATION_H
#endif // USER_API
