//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Scattering/IDecoratableBorn.cpp
//! @brief     Implements interface class IDecoratableBorn.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Scattering/IDecoratableBorn.h"
#include "Base/Vector/RotMatrix.h"
#include "Sample/Scattering/FormFactorDecoratorPositionFactor.h"
#include "Sample/Scattering/FormFactorDecoratorRotation.h"
#include "Sample/Scattering/Rotations.h"

IDecoratableBorn::IDecoratableBorn() = default;

IDecoratableBorn::IDecoratableBorn(const NodeMeta& meta, const std::vector<double>& PValues)
    : IFormFactor(meta, PValues)
{
}

IDecoratableBorn::~IDecoratableBorn() = default;

IDecoratableBorn* IDecoratableBorn::createTransformedFormFactor(const IDecoratableBorn& formfactor,
                                                                const IRotation* rot,
                                                                R3 translation)
{
    std::unique_ptr<IDecoratableBorn> P_fftemp, P_result;
    if (rot && !rot->isIdentity())
        P_fftemp = std::make_unique<FormFactorDecoratorRotation>(formfactor, rot->getRotMatrix());
    else
        P_fftemp.reset(formfactor.clone());
    if (translation != R3())
        P_result = std::make_unique<FormFactorDecoratorPositionFactor>(*P_fftemp, translation);
    else
        std::swap(P_fftemp, P_result);
    return P_result.release();
}
