//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Scattering/IDecoratableBorn.h
//! @brief     Defines interface IDecoratableBorn.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef USER_API
#ifndef BORNAGAIN_SAMPLE_SCATTERING_IDECORATABLEBORN_H
#define BORNAGAIN_SAMPLE_SCATTERING_IDECORATABLEBORN_H

#include "Sample/Scattering/IFormFactor.h"

//! Abstract base class for Born form factors.
//!
//! In contrast to the generic IFormFactor, a Born form factor does not depend
//! on the incoming and outgoing wave vectors ki and kf, except through their
//! difference, the scattering vector q=ki-kf.

class IDecoratableBorn : public IFormFactor {
public:
    IDecoratableBorn();
    IDecoratableBorn(const NodeMeta& meta, const std::vector<double>& PValues);
    ~IDecoratableBorn() override;

    IDecoratableBorn* clone() const override = 0;

    //! Returns scattering amplitude for complex scattering wavevector q=k_i-k_f.
    //! This method is public only for convenience of plotting form factors in Python.
    virtual complex_t formfactor(C3 q) const = 0;

    static IDecoratableBorn* createTransformedFormFactor(const IDecoratableBorn& formfactor,
                                                         const IRotation* rot, R3 translation);
};

#endif // BORNAGAIN_SAMPLE_SCATTERING_IDECORATABLEBORN_H
#endif // USER_API
