//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Scattering/IFormFactor.cpp
//! @brief     Implements interface class IFormFactor.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Scattering/IFormFactor.h"
#include "Base/Util/PyFmt.h"
#include "Base/Util/StringUtils.h"
#include "Base/Vector/WavevectorInfo.h"
#include <memory>
#include <utility>

IFormFactor::IFormFactor(const NodeMeta& meta, const std::vector<double>& PValues)
    : INode(meta, PValues), m_nodeMeta(meta)
{
}

std::string IFormFactor::shapeName() const
{
    if (className().substr(0, 10) == "FormFactor")
        return className().substr(10);
    return className();
}

Eigen::Matrix2cd IFormFactor::thePolFF(const WavevectorInfo&) const
{
    // Throws to prevent unanticipated behaviour
    throw std::runtime_error("IFormFactor::evaluatePol: is not implemented by default");
}

std::string IFormFactor::pythonConstructor() const
{
    std::vector<std::pair<double, std::string>> arguments;
    for (size_t i = 0; i < m_nodeMeta.paraMeta.size(); i++)
        arguments.emplace_back(m_P[i], m_nodeMeta.paraMeta[i].unit);

    return Py::Fmt::printFunction(className(), arguments);
}

double IFormFactor::volume() const
{
    auto zero_wavevectors = WavevectorInfo::makeZeroQ();
    return std::abs(theFF(zero_wavevectors));
}
